package calcolo_media;

/**
 * Oggetto che produce i dati che saranno poi recuperati dall'oggetto
 * Consumatore.
 *
 * @author <a href="mailto:ervincio@tiscali.it">Vincenzo Manzoni</a>
 */

public class Produttore extends Thread {
        /** Variabile per la condivisione dei dati Produttori - Consumatori. */
        private Buffer data;

        /** Tempo di ritardo tra l'invio di un dato e l'altro. */
        private int sleep_time;


        /**
         * Istanzia un Produttore, attribuendogli un nome, una variabile
         * per lo scambio di dati con i consumatori e un tempo di ritardo.
         *
         * @param name Nome identificativo del Produttore.
         *             Dovrebbe essere nelle forma PRODUTTOREn, dove
         *             n rappresenta un numero naturale.
         * @param data Variabile, di tipo Buffer, per lo scambio di dati con i
         *             consumatori.
         * @param sleep_time Tempo di ritardo.
         */
        public Produttore(ThreadGroup tg, String name, Buffer data, int sleep_time) {
                super(tg, name);
                this.data = data;
                this.sleep_time = sleep_time;
        }


        /**
         * Genera infiniti oggetti di tipo Integer, di valore intero crescente,
         * e li mette nel buffer.
         * Dopo averli messi nel Buffer, manda un messaggio di <code>notify()</code>
         * ai consumatori e stampa a video il valore intero dell'oggetto creato.
         *
         * <p>Tra un oggetto e l'altro, trascorre un tempo pari al tempo di ritardo
         * passato al costruttore.</p>
         *
         */
        public void run() {
                int i = 0;

                /*
                 * Variabile che mi determina lo stato di funzionamento del thread.
                 * Di default  messa a true
                 */
                boolean running = true;

                /*
                 * Ciclo che termina, o se la variabile running assume il valore false,
                 * oppure se la chiamata al metodo isInterrupted ritorna true.
                 *
                 * La variabile running pu assumere il valore false solo se il thread
                 * corrente viene interrotto mentre  in sleep() (vedere il blocco
                 * catch()).
                 *
                 * isInterrupted() ritorna false se il thread viene interrotto durante
                 * la normale esecuzione dalla chiamata al metodo interrupt() (per
                 * questo vedi il main()).
                 */
                while(running && !isInterrupted()) {

                        /*
                         * Blocco sincronizzato.
                         * Il motivo  che utilizzando direttamente la notify(), 
                         * necessario prima ottenere il lock sull'oggetto, altrimenti
                         * si causa un'eccezione del tipo IllegalMonitorStateException,
                         * che dar il poco intuitivo messaggio "current thread not owner".
                         */
                        synchronized(data) {
                                // Aggiungo l'oggetto al Buffer
                                data.add(new Integer(i));

                                //Stampa il proprio nome e il valore dell'intero messo nel buffer
                                System.out.println("[" + getName() +"]: Ho messo nel Buffer " + i);

                                // Notifico l'oggetto in wait() che il dato  pronto.
                                data.notifyAll();
                        }

                        i++;
                        try {
                                // Aspetto un tempo pari a sleep_time
                                Thread.sleep(sleep_time);
                        }
                        catch(InterruptedException ie) {

                                /*
                                 * Se ricevo un messaggio di interruzione del thread _mentre sono
                                 * in sleep_, viene generata un'eccezione (solo in questo caso,
                                 * in quanto se ricevo il messaggio di interruzione
                                 * mentre il thread non  sospeso mi viene settato un
                                 * flag che posso leggere attraverso il metodo isInterrupted()).
                                 * La intercetto e setto a false la variabile che mi determina lo
                                 * stato di funzionante del thread. In questo modo lo interrompo.
                                 */
                                running = false;
                        }
                }
        }
}
